<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);
/**
 * 2015-2016 Perzonalization
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@perzonalization.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Perzonalization to newer
 * versions in the future. 
 *
 *  @author    Perzonalization <team@perzonalization.com>
 *  @copyright 2015-2016 Perzonzalization
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 * International Registered Trademark & Property of PrestaShop SA
 * The class overloads 2 methods and adds a new one 
 */

require_once(_PS_MODULE_DIR_ . 'perzonalization' . DIRECTORY_SEPARATOR . 'perzonalization.php');

class PerzonalizationWebServiceModuleFrontController extends FrontController
{
    /**
     * @var Module
     */
    public $module;
    
    public function __construct()
    {
        $this->controller_type = 'modulefront'; //instead of standard controllers’ front
        
        $this->module = Module::getInstanceByName('perzonalization'); // an instance of the module is created
        
        if (!$this->module->active) {
            Tools::redirect('index'); //if the module is inactive, the user will be redirected to index.php
        }
        
        $this->page_name = 'module-' . $this->module->name . '-' . Dispatcher::getInstance()->getController();
        //for example: module-belvg_storelocator-search

        parent::__construct();
    }
    
    
    public function preProcess()
    {
        
    }
    
    public function initContent()
    {
        $id_lang = $this->context->language->id;
        $token = Configuration::get('PERZONALIZATION_TOKEN');
        
        // check the code
        if (Tools::getValue('accessToken') && Tools::getValue('accessToken') == $token) {
            if (Tools::getvalue('shopDetails')) {
                $ar             = array();
                $ar['email']    = (string) Configuration::get('PS_SHOP_EMAIL');
                $ar['country']  = $this->context->country->iso_code;
                $ar['currency'] = $this->context->currency->iso_code;
                
                $this->ajaxDie(Tools::jsonEncode($ar));
            }
            
            if (Tools::getValue('products')) {
                
                $per_page = 0;
                $page = 0;
                
                if(Tools::getValue('itemsPerPage')) {
                    $per_page = Tools::getValue('itemsPerPage');
                }
                if(Tools::getValue('pageIndex')) {
                    $page = Tools::getValue('pageIndex');
                }
                
                $manufacturers = Manufacturer::getManufacturers(false, $id_lang);
                
                //make array of man names for easy access
                $manufacturer_name = array();
                foreach ($manufacturers as $key => $manufacturer) {
                    $manufacturer_name[$manufacturer['id_manufacturer']] = $manufacturer['name'];
                }
                $product_array = array();
                
                //retrieve all products
                if (!Tools::getValue('id')) {
                    $products = Product::getProducts($id_lang, (int)(($page-1)*$per_page), (int)$per_page, 'id_product', 'ASC');
                    foreach ($products as $key => $product) {
                        $prod                = new Product($product['id_product'], false, $id_lang);
                        $product_array[$key] =
                            Perzonalization::convertProductArray($prod, $manufacturer_name, $id_lang);
                    }
                } else {
                    $prod          = new Product((int) Tools::getValue('id'), false, $id_lang);
                    $product_array = Perzonalization::convertProductArray($prod, $manufacturer_name, $id_lang);
                }
                
                $this->ajaxDie(Tools::jsonEncode($product_array));
            }
            
            if (Tools::getValue('orders')) {
                $order_array = array();
                //retrieve all orders
                if (!Tools::getValue('id')) {
                    $date_to   = date('Y-m-d');
                    $date_from = '0000-00-00';
                    $page = "";
                    $per_page= "";
                    if (Tools::getValue('endDate')) {
                        $date_to = Tools::getValue('endDate');
                    }
                    if (Tools::getValue('startDate')) {
                        $date_from = Tools::getValue('startDate');
                    }
                    if(Tools::getValue('itemsPerPage')) {
                        $per_page = Tools::getValue('itemsPerPage');
                    }
                    if(Tools::getValue('pageIndex')) {
                        $page = Tools::getValue('pageIndex');
                    }
                    
                    $orders_ids = self::getOrdersIdByDateWithLimits($date_from, $date_to,$page,$per_page);
                    
                    foreach ($orders_ids as $id) {
                        $order         = new Order($id, $id_lang);
                        $order_array[] = self::convertOrderArray($order, $id_lang);
                    }
                } else {
                    $order         = new Order(Tools::getValue('id'), $id_lang);
                    $order_array[] = self::convertOrderArray($order, $id_lang);
                }
                
                $this->ajaxDie(Tools::jsonEncode($order_array));
            }
			
            if (Tools::getValue('orders_count')) {
                $date_to   = date('Y-m-d');
                    $date_from = '0000-00-00';
                    if (Tools::getValue('endDate')) {
                        $date_to = Tools::getValue('endDate');
                    }
                    if (Tools::getValue('startDate')) {
                        $date_from = Tools::getValue('startDate');
                    }
                    $orders_ids = self::getOrdersIdByDateWithLimits($date_from, $date_to);
                    $order_num = count($orders_ids);
                    $this->ajaxDie(Tools::jsonEncode($order_num));
            }
            
            if (Tools::getValue('products_count')) {
                    $date_to   = date('Y-m-d');
                    $date_from = '0000-00-00';
                    if (Tools::getValue('endDate')) {
                        $date_to = Tools::getValue('endDate');
                    }
                    if (Tools::getValue('startDate')) {
                        $date_from = Tools::getValue('startDate');
                    }
                    $sql = 'SELECT id_product FROM '._DB_PREFIX_.'product WHERE date_add > \''.pSQL($date_from).'\' AND date_add < \''.pSQL($date_to).'\'';
                    $prod_ids = Db::getInstance()->executeS($sql);
                    $this->ajaxDie("{\"count\":".count($prod_ids)."}");
            }

            // get total customers
            if (Tools::getValue('customers_count')) {
                die(Tools::jsonEncode(array(
                    'count' => $this->module->getTotalCustomers()
                )));
            }

            // get customers' details
            if (Tools::getValue('customers')) {
                $page    = ((int)Tools::getValue('pageIndex') ?: 1);
                $perPage = ((int)Tools::getValue('itemsPerPage') ?: 25);

                die(Tools::jsonEncode(array(
                    'customers' => $this->module->getCustomersDetails($page, $perPage)
                )));
            }

            $this->ajaxDie('No request');
        } else {
            $this->ajaxDie('No token or wrong token');
        }
    }
    
    
    public static function convertOrderArray(Order &$order, $id_lang)
    {
        $order_array = array();
        
        $order_array['id']     = $order->id;
        $order_array['userId'] = $order->id_customer;
        
        $the_date = strtotime($order->date_add);
        date_default_timezone_set("UTC");
        $order_array['time'] = date("Y-m-d H:i:s", $the_date);
        $products            = $order->getProducts();
        
        $product_arr = array();
        
        
        $attr_groups         = AttributeGroup::getAttributesGroups($id_lang);
        $attr_groups_rebuilt = array();
        //foreach($attr_groups as $key => $group){ //validator complaint: $key is unused
        foreach ($attr_groups as $group) {
            $attr_groups_rebuilt[$group['id_attribute_group']]['name'] = $group['public_name'];
        }
        
        $it = 0;
        //foreach ($products as $key => $product) { //validator complaint: unused variable
        foreach ($products as $product) {
            $product_arr[$it]['id']       = $product['product_id'];
            $product_arr[$it]['price']    = Tools::displayPrice($product['product_price'], (int) $order->id_currency);
            $product_arr[$it]['quantity'] = $product['product_quantity'];
            
            $combination = new Combination($product['product_attribute_id']);
            $attributes  = $combination->getAttributesName($id_lang);
            
            $product_arr[$it]['attributes'] = array();
            foreach ($attributes as $attr) {
                $attribute = new Attribute($attr['id_attribute']);
                $product_arr[$it]['attributes'][$attr_groups_rebuilt[$attribute->id_attribute_group]['name']] =
                    $attr['name'];
            }
            $it++;
        }
        
        $order_array['products'] = $product_arr;
        
        return $order_array;
    }
    
    
    public static function getOrdersIdByDateWithLimits($date_from, $date_to, $page="", $per_page="")
	{
		$sql = 'SELECT `id_order`
				FROM `'._DB_PREFIX_.'orders`
				WHERE DATE_ADD(date_upd, INTERVAL -1 DAY) <= \''.pSQL($date_to).'\' AND date_upd >= \''.pSQL($date_from).'\'
					'.Shop::addSqlRestriction()
					.(($page && $per_page) ? ' LIMIT '.(int)(($page-1)*$per_page).','.(int)$per_page: '');
		$result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

		$orders = array();
		foreach ($result as $order)
			$orders[] = (int)($order['id_order']);
		return $orders;
	}
}
