<?php
/**
 * 2015-2016 Perzonalization
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@perzonalization.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Perzonalization to newer
 * versions in the future.
 *
 *  @author    Perzonalization <team@perzonalization.com>
 *  @copyright 2015-2016 Perzonzalization
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class Perzonalization extends Module
{
    protected $config_form = false;
    public $guid;
    public $token;
    public $order_id;
    public function __construct()
    {
        $this->name          = 'perzonalization';
        $this->tab           = 'advertising_marketing';
        $this->version       = '1.5.1';
        $this->author        = 'Perzonalization';
        $this->need_instance = 0;
        $this->guid =  strtolower(Configuration::get('PERZONALIZATION_GUID'));
        $this->token =  strtolower(Configuration::get('PERZONALIZATION_TOKEN'));
        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Perzonalization - Recommendations in Real Time');
        $this->description = $this->l('Personalized recommendations. Boost Sales by 25%. Used by Enterprises. One Click Set-up');

        $this->confirmUninstall =
            $this->l('Is there anything we could help? Please tell us what we did wrong and we will fix it!');
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
		Configuration::updateValue('PERZONALIZATION_LIVE_MODE', false);

		include(dirname(__FILE__) . '/sql/install.php');

		$token = self::generateToken(32);
		$existing_guid =  strtolower(Configuration::get('PERZONALIZATION_GUID'));
		if(!$existing_guid)
			$guid = strtolower(self::generateGuid());

		Configuration::updateValue('PERZONALIZATION_TOKEN', $token);

		if($existing_guid) {
			$guid = $existing_guid;
		} else {
			Configuration::updateValue('PERZONALIZATION_GUID', $guid);
		}

		$link       = new Link();
		$moduleLink = $link->getModuleLink('perzonalization', 'webservice');

		$shop_email = null;
		if(Configuration::get('PS_SHOP_EMAIL')){
			$shop_email = Configuration::get('PS_SHOP_EMAIL');
		}
		$shop_phone = null;
		if(Configuration::get('PS_SHOP_PHONE')){
			$shop_phone = Configuration::get('PS_SHOP_PHONE');
		}
		$shop_city = null;
		if(Configuration::get('PS_SHOP_CITY')){
			$shop_city = Configuration::get('PS_SHOP_CITY');
		}
		$shop_name = null;
		if(Configuration::get('PS_SHOP_NAME')){
			$shop_name = Configuration::get('PS_SHOP_NAME');
		}
		else{
			$shop_name = Configuration::get('PS_SHOP_DOMAIN');
		}		

		//admin url
		try{
			$admin = explode(DIRECTORY_SEPARATOR,_PS_ADMIN_DIR_);
			$admin_folder = array_pop((array_slice($admin, -1)));
			$admin_url =_PS_BASE_URL_.__PS_BASE_URI__.$admin_folder;
		}
		catch(Exception $e){
			$admin_url = null;
		}

		$defaultLanguage = intval(Configuration::get('PS_LANG_DEFAULT'));		
		$defaultLanguage_iso = Language::getIsoById($defaultLanguage);
		$defaultCurrency_iso = Currency::getDefaultCurrency()->iso_code;

		$replace    = array(
		 'http://',
		 'https://'
		);
		$shop_url = str_replace($replace, '', _PS_BASE_URL_);

		$data       = array(
			"accessToken" => $token,
			"city" => $shop_city,
			"country" => $this->context->country->iso_code,
			"currency" => $defaultCurrency_iso,
			"displayName"=>$shop_name,
			"email" => $shop_email,
			"language" => $defaultLanguage_iso,
			"modulePath" => $moduleLink,
			"phone" => $shop_phone,
			"pluginVersion" => $this->version,
			"url" => $shop_url,
			"platformVersion" => _PS_VERSION_,
			"adminUrl" => $admin_url
		);

		$result =
			Perzonalization::doPut('http://api.perzonalization.com/stores/prestashop.'.urlencode($guid), $data);

		if (!$result) {
			return false;
		}

		//Thanks! - http://doc.prestashop.com/display/PS15/Hooks+in+PrestaShop+1.5
		return parent::install()
			&& $this->registerHook('header')
			&& $this->registerHook('backOfficeHeader')
			&& $this->registerHook('displayFooter')
			&& $this->registerHook('displayHome')
			&& $this->registerHook('displayFooterProduct')
			&& $this->registerHook('displaySearch')
			&& $this->registerHook('displayShoppingCartFooter')
			&& $this->registerHook('displayShoppingCart')
			&& $this->registerHook('displayOrderConfirmation')
			&& $this->registerHook('displayLeftColumn')
			&& $this->registerHook('displayHeader')
			&& $this->registerHook('actionCartSave');
    }

    public function uninstall()
    {
        Configuration::deleteByName('PERZONALIZATION_LIVE_MODE');
        include(dirname(__FILE__) . '/sql/uninstall.php');

        /* send uninstall request to perzonalization service */
        $replace   = array(
            'http://',
            'https://'
        );
        $guid =  Configuration::get('PERZONALIZATION_GUID');
        Perzonalization::doDelete('http://api.perzonalization.com/stores/prestashop.'.$guid);

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        //die(Configuration::get('PS_SHOP_EMAIL'));
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool) Tools::isSubmit('submitPerzonalizationModule')) == true) {
            $this->postProcess();
        }

        $replace = array(
            'http://',
            'https://'
        );

        $shop_name = str_replace($replace, '', _PS_BASE_URL_);

        $url = "//my.perzonalization.com/stores/prestashop.".urldecode($this->guid)."/config?at=".urldecode($this->token);

        $this->context->smarty->assign('module_dir', $this->_path);
        $this->context->smarty->assign('url', $url);

        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        return $output;
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar             = false;
        $helper->table                    = $this->table;
        $helper->module                   = $this;
        $helper->default_form_language    = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier    = $this->identifier;
        $helper->submit_action = 'submitPerzonalizationModule';
        $helper->currentIndex  =
            $this->context->link->
                getAdminLink('AdminModules', false)
                . '&configure='
                . $this->name
                . '&tab_module='
                . $this->tab
                . '&module_name='
                . $this->name;
        $helper->token         = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(),
            /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array(
            $this->getConfigForm()
        ));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Live mode'),
                        'name' => 'PERZONALIZATION_LIVE_MODE',
                        'is_bool' => true,
                        'desc' => $this->l('Use this module in live mode'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        )
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-envelope"></i>',
                        'desc' => $this->l('Enter a valid email address'),
                        'name' => 'PERZONALIZATION_ACCOUNT_EMAIL',
                        'label' => $this->l('Email')
                    ),
                    array(
                        'type' => 'password',
                        'name' => 'PERZONALIZATION_ACCOUNT_PASSWORD',
                        'label' => $this->l('Password')
                    )
                ),
                'submit' => array(
                    'title' => $this->l('Save')
                )
            )
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'PERZONALIZATION_LIVE_MODE' => Configuration::get('PERZONALIZATION_LIVE_MODE', true),
            'PERZONALIZATION_ACCOUNT_EMAIL' =>
                Configuration::get('PERZONALIZATION_ACCOUNT_EMAIL', 'team@perzonalization.com'),
            'PERZONALIZATION_ACCOUNT_PASSWORD' => Configuration::get('PERZONALIZATION_ACCOUNT_PASSWORD', null)
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }
	
	public function getLastProduct($cartId)
	{
		$sql = '
			SELECT cp.`id_product`, cp.`id_product_attribute`, p.`id_category_default`, cp.`quantity`
			FROM `'._DB_PREFIX_.'cart_product` cp
			JOIN `'._DB_PREFIX_.'product` p ON (cp.`id_product` = p.`id_product`)
			WHERE `id_cart` = '.(int)($cartId).'
			ORDER BY cp.`date_add` DESC';
		$result = Db::getInstance()->getRow($sql);
		if ($result AND isset($result['id_product']) AND $result['id_product'])
			return $result;
		return false;
	}

	public function prettyDump($data)
	{
		highlight_string("<?php\n\$data =\n" . var_export($data, true) . ";\n?>");
    }
		
	public function hookActionCartSave($params) {
		// $cartId = $params['cart']->id;
		// $items2 = $this->getLastProduct($cartId);
	}

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        // $this->context->controller->addJS($this->_path.'/views/js/front.js');
        //$this->context->controller->addCSS($this->_path.'/views/css/front.css');
    }

    private function injectPerzonalizationContent($is_cart = false)
    {
        $id_lang = $this->context->language->id;
        $output = "<div id='perzonalization' class='perzonalization'></div>";

        $output .= '
		<!--PERZONALIZATION-START Do not modify or delete this comment-->
		<script>' . 'var detailsForPrz =  ';

        //$isProductPage = null;
        $prod_array = array();

        $prod_array['instanceGuid'] = $this->guid;

        /*
         *  'basket', 'filter', 'home', 'product', 'sales', 'search'
         */
        //$page_name = "n/a";
        $pagetypes = array(
            'index'=>'home',
            'order'=>'basket',
            'category'=>'filter',
            'cart'=>'basket',
            'order-confirmation'=>'sales',
            '3'=>'',
            '4'=>'',
            '5'=>'',
            '6'=>''
        );

        if(isset($pagetypes[$this->context->controller->php_self])) {
            $prod_array['pageType'] = $pagetypes[$this->context->controller->php_self];
        } else {
            $prod_array['pageType'] = $this->context->controller->php_self;
        }


        $id_category = Tools::getValue('id_category');
        if ($id_category) {
            $category         = new Category($id_category, $this->context->language->id);
            $categories_array = $category->getParentsCategories($this->context->language->id);
            $cat_ar           = array();

            foreach ($categories_array as $cat) {
                $cat_ar[] = $cat['name'];
            }

            $prod_array['categories'] = $cat_ar;
        }


        //return user id
        if ($this->context->customer->isLogged(true)) {
            $prod_array['userId'] = $this->context->customer->id;
        }
		else{
			$prod_array['userId'] = null;
		}
		
		$prod_array['currency'] = $this->context->currency->iso_code;
        $prod_array['language'] = $this->context->language->iso_code;
        
        $replace                    = array(
            'http://',
            'https://'
        );
        $shop_name                  = str_replace($replace, '', _PS_BASE_URL_);
        $prod_array['instanceName'] = $shop_name;

        $output .= Tools::jsonEncode($prod_array);
        $output .= '; ' . '</script>';

		//basketDetailsForPrz in every page ------------START
		
		
		$cart_array = array();
		$output .= '<script>' . 'var basketDetailsForPrz =  ';
		$products = $this->context->cart->getProducts();
	   /* $cart_product_ids = array();
		foreach($cart_products_array as $cart_product) {
			$cart_product_ids[] = $cart_product['id_product'];
		}
		$prod_array['ids']=implode(',',$cart_product_ids);*/

		$product_arr = array();


		$attr_groups         = AttributeGroup::getAttributesGroups($id_lang);
		$attr_groups_rebuilt = array();
		//foreach($attr_groups as $key => $group){ //validator complaint: $key is unused
		foreach ($attr_groups as $group) {
			$attr_groups_rebuilt[$group['id_attribute_group']]['name'] = $group['public_name'];
		}

		//die(print_r($products));

		$it = 0;
		//foreach ($products as $key => $product) { //validator complaint: unused variable
		foreach ($products as $product) {
			$product_arr[$it]['id']       = $product['id_product'];
			$product_arr[$it]['price']    = Tools::displayPrice($product['price'], (int) $this->context->currency->id);
			$product_arr[$it]['quantity'] = $product['quantity'];

			$combination = new Combination($product['id_product_attribute']);
			$attributes  = $combination->getAttributesName($id_lang);

			$product_arr[$it]['attributes'] = array();
			foreach ($attributes as $attr) {
				$attribute = new Attribute($attr['id_attribute']);
				$product_arr[$it]['attributes'][$attr_groups_rebuilt[$attribute->id_attribute_group]['name']] =
					$attr['name'];
			}
			$it++;
		}

		$cart_array['products'] = $product_arr;
		$output .= Tools::jsonEncode($cart_array);

		$output .= '; ' . '</script>';
        
		
		
		//basketDetailsForPrz in every page ------------END
		
        if($prod_array['pageType'] == 'sales') { //order confirmation
            $order_array = array();
            $output .= '<script>' . 'var purchaseDetailsForPrz =  ';
            $order = new Order($this->order_id);
            //$order = new Order('8');
            $products = $order->getProducts();
            $order_array['transactionId']=$order->id;
            $order_array['userId']=$order->id_customer;
            $products            = $order->getProducts();
            $product_arr = array();
            $attr_groups         = AttributeGroup::getAttributesGroups($id_lang);
            $attr_groups_rebuilt = array();
            //foreach($attr_groups as $key => $group){ //validator complaint: $key is unused
            foreach ($attr_groups as $group) {
                $attr_groups_rebuilt[$group['id_attribute_group']]['name'] = $group['public_name'];
            }

            $it = 0;
            //foreach ($products as $key => $product) { //validator complaint: unused variable
            foreach ($products as $product) {				
                $product_arr[$it]['id']       = $product['product_id'];
                $product_arr[$it]['price']    = Tools::displayPrice($product['product_price'], (int) $order->id_currency);
                $product_arr[$it]['quantity'] = $product['product_quantity'];

                $combination = new Combination($product['product_attribute_id']);
                $attributes  = $combination->getAttributesName($id_lang);

                $product_arr[$it]['attributes'] = array();
                foreach ($attributes as $attr) {
                    $attribute = new Attribute($attr['id_attribute']);
                    $product_arr[$it]['attributes'][$attr_groups_rebuilt[$attribute->id_attribute_group]['name']] =
                        $attr['name'];
                }
                $it++;
            }

            $order_array['productData'] = $product_arr;
            $output .= Tools::jsonEncode($order_array);

            $output .= '; ' . ' var _przq = _przq || []; _przq.push(purchaseDetailsForPrz)</script>';
        }

        /*var purchaseDetailsForPrz = {
  transactionId: '4089', userId: '19229', //unique id of the user in the system, who made the purchase
  productData: [
    { id: '93724', price: '49.95', quantity: '2' }, //purchased price of the product
    { id: '144983', price: '19.99', quantity: '1', size: 'XL' } //size to be provided if applicable and available
  ]};
var _przq = _przq || []; _przq.push(purchaseDetailsForPrz);*/

        

        if ($idProduct = Tools::getValue('id_product')) { //product details

            $output .= '<script>' . 'var productDetailsForPrz = ';
            $product = new Product($idProduct, false, $this->context->language->id);

            $manufacturers = Manufacturer::getManufacturers(false, $this->context->language->id);

            //make array of man names for easy access
            $manufacturer_name = array();
            //foreach($manufacturers as $key => $manufacturer)//validator complaint: unused var
            foreach ($manufacturers as $manufacturer) {
                $manufacturer_name[$manufacturer['id_manufacturer']] = $manufacturer['name'];
            }

            $prod_array = self::convertProductArray($product, $manufacturer_name, $this->context->language->id);

            $output .= Tools::jsonEncode($prod_array);

            $output .= '; ' . '</script>';

            /*
             * attributes: [
    { 'name': 'tag', 'value': 'street style' }, //'tags' are to be added like this -- see the very bottom of 'Information' page in backOffice
    { 'name': 'tag', 'value': 'pastel colors' },
    { 'name': 'compositions', 'value': 'cotton' }, //'features' are to be added like this -- see the 'Features' page in backOffice
    { 'name': 'styles', 'value': 'casual' },
    { 'name': 'properties', 'value': 'short sleeve' },
    { 'name': 'size', 'value': 'S' }, //'available attributes' are to be added like this
    { 'name': 'color', 'value': 'orange' },
    //...
  ],
  campaign: { id: '12345', name: 'Buy 3 Pay For 2' }, //:null if not a campaign product
  canonicalUrl: '//dev.gurabo.ru/evening-dresses/4-printed-dress.html'; //RENAME: url --> canonicalUrl -- canonical url of the product
  categories: ['women', 'tops', 't-shirts'], //RENAME: associatedCategories --> categories -- all categories until the leaf node -- SEE: Update 4
  description: "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Nulla in purus ut nisi posuere rutrum. Nullam id laoreet mauris. Donec varius libero sem, non convallis felis aliquet ac. Nam in eros sit amet justo lacinia convallis quis id tellus.", //SEE: Update 5
  id: "1183",
  manufacturer: "Zara",
  name: "Printed Dress",
  onSale: true, //RENAME: available -> onSale -- whether the product in on sale (=enabled and available for order)
  regularPrice: "250.00", //RENAME: previousPrice -> regularPrice -- previous sales price (if product is on sale). Send null if n/a.
  salesPrice: "199.99", //RENAME: price -> salesPrice -- current sales price
  thumbnailUrl: '//dev.gurabo.ru/10-large_default/printed-dress.jpg' //RENAME: thumbUrl -> thumbnailUrl
  variantsOnSale: [ //SEE: Update 2
    { 'name': 'size', 'value': 'XS', 'variantId': '8681051375846' },
    { 'name': 'size', 'value': 'S', 'variantId': '8681051375853' },
    { 'name': 'size', 'value': 'L', 'variantId': '8681051375877' }
    //...
  ]
             */

        }

        if($prod_array['pageType'] == 'filter') {

            $output .= '<script>' . 'var filterDetailsForPrz =  ';

            //for 1.7 and later - [listing][products], 1.6 and before - [products]

            $products = array();

            $allvars = $this->context->smarty->getTemplateVars();
            //print_r($allvars);

            if(version_compare(_PS_VERSION_, '1.7.0', '>=')){
                $products = $this->context->smarty->getTemplateVars('listing');
                $products = $products['products'];
            } else {
                $products = $this->context->smarty->getTemplateVars('products');
            }

            $product_ids = array();
            foreach($products as $prod) {
                $product_ids[] = $prod['id_product'];
            }
			if(count($product_ids) > 3){
				//get last 3 products to exclude
				$product_ids = array_slice($product_ids, -3, 3, false);
			}
			
            $prod_array['ids'] = $product_ids;
            $output .= Tools::jsonEncode($prod_array);
            $output .= '; ' . '</script>';

        }
		$output .= '
		<!--PERZONALIZATION-END Do not modify or delete this comment-->';
		
        return $output;
    }

    public function hookDisplayFooter()
    {
        // $page_name = "";
        // if (isset($this->context->controller->php_self)) {
            // $page_name = $this->context->controller->php_self;
        // }
        // if ($page_name == 'search') {
            // return $this->injectPerzonalizationContent();
        // } else {
            // return '<!-- THIS IS LEFT COLUMN DISPLAY HOOK -->';
        // }
		return '<!-- THIS IS FOOTER DISPLAY HOOK -->';
    }

    public function hookDisplayHome()
    {
        return $this->injectPerzonalizationContent();
    }

    public function hookDisplayShoppingCartFooter()
    {
        return $this->injectPerzonalizationContent(true);
    }
	
	public function hookDisplaySearch()
    {
        return $this->injectPerzonalizationContent(false);
    }

    public function hookDisplayShoppingCart()
    {
        //return $this->injectPerzonalizationContent(true);
    }

    public function hookDisplayFooterProduct()
    {
        return $this->injectPerzonalizationContent();
    }

    public function hookDisplayOrderConfirmation($params)
    {
        $this->order_id = Tools::getValue('id_order');
        return $this->injectPerzonalizationContent();
    }

    public function hookDisplayLeftColumn()
    {
		$page_name = "";
		if (isset($this->context->controller->php_self)) {
			$page_name = $this->context->controller->php_self;
		}
		//if ($page_name == 'search' || $page_name == 'category') {
		if ($page_name == 'category') {
			return $this->injectPerzonalizationContent();
		} else {
			return '<!-- THIS IS LEFT COLUMN DISPLAY HOOK -->';
		}
    }

    public function hookDisplayHeader()
    {
        //Thanks! - http://nemops.com/columns-for-each-prestashop-page/#.VqsvUbB94Wo
        //Thanks! - https://www.prestashop.com/forums/topic/396501-php-function-to-get-equivalent-to-smarty-page-name/

        $page_name = "";
        if (isset($this->context->controller->php_self)) {
            $page_name = $this->context->controller->php_self;
        }

        $output = "<script type='text/javascript' async>(function (w, d, n, i, a, l, s, r, c) { r = Math.round(Math.random() * 1e4); c = d.getElementById(i); if (!c) { s = d.createElement(n); s.type = 'text/javascript'; s.id = i; s.src = '//' + a + '?rnd=' + r; s.async = 1; l = d.getElementsByTagName(n)[0]; l.parentNode.insertBefore(s, l); } if (c) { runPRZPlugin(true); } })(window, document, 'script', 'prz_loader', 'cdn.perzonalization.com/js/loader/prestashop.loader.js'); 
//---  </script>";


        return $output;
    }

    private static function doPut($url, $data)
    {
        if ($ch = curl_init()) {
            $data_string = Tools::jsonEncode($data);

            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',
                'Content-Length: ' . Tools::strlen($data_string)
            ));

            curl_exec($ch);
            $info = curl_getinfo($ch);
            curl_close($ch);

            if (!empty($info['http_code']) && $info['http_code'] == '201') {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    private static function doDelete($url)
    {
        if ($ch = curl_init()) {
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "DELETE");
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HEADER, false);
            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json'
            ));

            curl_exec($ch);
            $info = curl_getinfo($ch);
            curl_close($ch);

            if (!empty($info['http_code']) && $info['http_code'] == '200') {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    public static function generateToken($length = 10)
    {
        $characters       = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = Tools::strlen($characters);
        $randomString     = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    public static function convertProductArray(Product &$prod, array $manufacturer_name, $id_lang)
    {
		$default_language_id = (int)Configuration::get('PS_LANG_DEFAULT');
		//$this->context->currency = new Currency((int)Configuration::get('PS_CURRENCY_DEFAULT'));
		$id_lang = $default_language_id;
		
        $link = new Link();

        $product_array       = array();
        $product_array['id'] = $prod->id;

        $row                = get_object_vars($prod);
        $row['id_product']  = $prod->id;
        $productsProperties = Product::getProductProperties($id_lang, $row);

        $product_array['available'] = false;

        //availability with stock
        if (isset($prod->available_for_order) && $prod->available_for_order && isset($prod->active) && $prod->active) {
            if ($productsProperties['allow_oosp'] || $productsProperties['quantity'] > 0) {
                $product_array['available'] = true;
            } elseif (
                    isset($productsProperties['quantity_all_versions'])
                    && $productsProperties['quantity_all_versions'] > 0
                ) {
                $product_array['available'] = true;
            } else {
                $product_array['available'] = false;
            }
        }
        //end availability

        //tags
        $tags = $prod->getTags($id_lang);

        $tags = explode(',',$tags);

        foreach($tags as $tag) {
            $product_array['attributes'][] = array (
                'name' => 'tag',
                'value' => $tag
            );
        }
        //end tags

        $product_array['canonicalUrl']      = $link->getProductLink($prod);
        $product_array['name']     = $prod->name;
        $cover                     = Product::getCover($prod->id);
        //ImageType::getFormatedName('large') instead of 'large_default';
		$formattedName = null;
		if(version_compare(_PS_VERSION_, '1.7.0', '>=')){
			$formattedName = ImageType::getFormattedName('large');
		}
		else{
			$formattedName = ImageType::getFormatedName('large');
		}

        $product_array['thumbnailUrl'] = str_replace(
            'http://',
            Tools::getShopProtocol(),
            Context::getContext()->link->getImageLink(
                $prod->link_rewrite,
                $cover['id_image'],
                $formattedName
            )
        );
		
        $current_price             = $prod->getPrice();
		$current_price = Tools::convertPrice($current_price, null, false); 
        $prev_price                = $prod->getPriceWithoutReduct();
		$prev_price = Tools::convertPrice($prev_price, null, false); 
		
        $product_array['salesPrice']    = Tools::displayPrice($current_price);

        if ($current_price != $prev_price) {
            $product_array['regularPrice'] = Tools::displayPrice($prev_price);
        } else {
            $product_array['regularPrice'] = null;
        }

        if (array_key_exists($prod->id_manufacturer, $manufacturer_name)) {
            $product_array['manufacturer'] = $manufacturer_name[$prod->id_manufacturer];
        } else {
            $product_array['manufacturer'] = null;
        }

        //get categories
        $category = new Category($prod->id_category_default, $id_lang);

        $categories_array = $category->getParentsCategories($id_lang);

        $cat_ar = array();

        foreach ($categories_array as $cat) {
            $cat_ar[] = $cat['name'];
        }

        $product_array['categories'] = $cat_ar;
        $features = $prod->getFrontFeatures($id_lang);

        if ($features) {
            $feat_array = array();

            $f = 0;
            foreach ($features as $feature) {
                $product_array['attributes'][] = array (
                    'name'=>$feature['name'],
                    'value'=>$feature['value']
                );
            }
        }
        else {
            $product_array['features'] = array();
        }

        //combinations
        $combinations = Product::getAttributesInformationsByProduct($prod->id);
        //die(print_r($combinations));

        if ($combinations) {
            $comb_array = array();

            foreach ($combinations as $key => $combination) {
                $comb_array[$combination['id_attribute']] = array( // for variantsOnSale usage
                  'group_name' =>  $combination['group']
                );

                $product_array['attributes'][] = array(
                    'name' => $combination['group'],
                    'value' => $combination['attribute']
                );
            }
        }
        //end combinations

        //get variants on sale
        $sp_ids = Db::getInstance()->executeS('SELECT id_product_attribute FROM '._DB_PREFIX_.'specific_price WHERE id_product='.$prod->id);
        if (count($sp_ids) > 0 ) {
            //select combinations on sale
            foreach($sp_ids as $sp_id) {
                if($sp_id['id_product_attribute'] > 0) { //otherwise it's just a product on sale
                    //die(print_r($sp_id));
                    $comb = new Combination($sp_id['id_product_attribute']);
                    $attributes = $comb->getAttributesName($id_lang);
                    foreach($attributes as $attribute) {
                        $product_array['variantsOnSale']['name'] = $comb_array[$attribute['id_attribute']]['group_name'];
                        $product_array['variantsOnSale']['value'] = $attribute['name'];
                        $product_array['variantsOnSale']['variantId'] = $sp_id['id_product_attribute'];
                    }
                }
            }
        }
        return $product_array;
    }

    public static function generateGuid()
    {
        if (function_exists('com_create_guid') === true)
        {
            return trim(com_create_guid(), '{}');
        }

        return sprintf('%04X%04X-%04X-%04X-%04X-%04X%04X%04X', mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(16384, 20479), mt_rand(32768, 49151), mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535));
    }

    /**
     * Get total customers from db
     */
    public function getTotalCustomers()
    {
        return Db::getInstance()->getValue('
            SELECT COUNT(`id_customer`) FROM `'. _DB_PREFIX_ .'customer`'
        );
    }

    /**
     * Get customers details
     */
    public function getCustomersDetails($page, $per_page)
    {
        $sql = 'SELECT c.`id_customer` as id, DATE_FORMAT(c.`date_add`, "%Y-%m-%dT%H:%i:%sZ") as createdAt, c.`email`, c.`firstname` as firstName, c.`lastname` as lastName, c.`active` as isActive, 
            (SELECT COUNT(`id_order`) FROM `'. _DB_PREFIX_ .'orders` od WHERE od.`id_customer` = c.`id_customer`) as ordersCount, 
            (SELECT ROUND(SUM(o.`total_paid` / o.`conversion_rate`), 2) FROM `'. _DB_PREFIX_ .'orders` o WHERE o.`id_customer` = c.`id_customer`) as totalSpent, 
            (SELECT concat(do.`id_order`, "#", DATE_FORMAT(do.`date_add`, "%Y-%m-%dT%H:%i:%sZ")) FROM `'. _DB_PREFIX_ .'orders` do WHERE do.`id_customer` = c.`id_customer` order by do.`id_order` DESC LIMIT 1) as lastOrderDetails 
            FROM '. _DB_PREFIX_ .'customer c 
            ORDER BY c.`id_customer` ASC LIMIT '. (int)(($page-1)*$per_page) .', '. (int)$per_page;
        $customers = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        
        // if no customer found
        if (!count($customers)) {
            return [];
        }

        // iterate through customers to get last order id and date
        foreach ($customers as &$customer) {
            $lastOrderId   = '';
            $lastOrderDate = null;
            // we need to split last order hack
            if (isset($customer['lastOrderDetails']) && !empty($customer['lastOrderDetails'])) {
                list($lastOrderId, $lastOrderDate) = explode('#', $customer['lastOrderDetails']);
            }

            $customer['lastOrderId']   = $lastOrderId;
            $customer['lastOrderDate'] = $lastOrderDate;
            // no longer need, just omit it
            unset($customer['lastOrderDetails']);
        }

        return $customers;
    }
}
